/* Fork Test: display args, open files, signals, etc.
 *
 * Simple as this program is, it has found bugs in the
 * way a number of programs fork off children.  To test
 * how a program is invoking its children, run this
 * program as a child.
 *
 * Generally, processes should be created with:
 *
 * - a reasonable arg count & list
 * - arg 0 should look like the name of the command
 *
 * - real and effective UIDs and GIDs should be reasonable.
 *   Beware setuid programs that fork children!
 *
 * - no pending alarm.  Version 7 apparently does not
 *   reset alarms upon an exec!
 *
 * - file descriptors 0 (STDIN), 1 (STDOUT), and 2 (STDERR)
 *   opened reasonably
 * - all other file descriptors closed (this program will
 *   describe all open channels)
 *
 * - all signals (except SIGKILL) set to SIG_DFL (this
 *   program will print all signals set otherwise)
 *
 * The output is fairly simple to understand.  When in
 * doubt, read the code (and a UNIX manual: exec(2),
 * getuid(2), alarm(2), signal(2), stat(2)).
 *
 * Room for Improvement:
 *
 * - strings should be printed in a way that shows funny characters.
 * - show misc. other bits of state
 *	- PID (who cares?)
 *	- umask
 *	- ulimit (System V)
 *	- stty settings of open TTYs
 *
 * Copyright (c) 1986 March 11  D. Hugh Redelmeier
 *
 * This program may be distributed and used without restriction.
 */

#include <stdio.h>

extern unsigned alarm();	/* should be unsigned, but may be int */

#include <sys/types.h>
#include <sys/stat.h>

struct stat sb;

#include <errno.h>
extern int errno;

#ifdef NO_STRERROR
extern char *sys_errlist[];
#define strerror(x)	sys_errlist[(x)]
#endif

#include <signal.h>

/*int (*signal())();*/

main(argc, argv, envp)
int argc;
char **argv, **envp;
{
	register int i;
	unsigned al = alarm(0);	/* get it while it is hot */

	printf("%d arg(s):", argc);
	for (i=0; i<argc; i++)
		if (argv[i] == NULL)
			printf(" NULL!");
		else
			printf(" \"%s\"", argv[i]);
	printf("\n");
	if (argv[argc] != NULL)
		printf("Arg list is not ended with a NULL!\n");

	printf("Real UID = %d, GID = %d; Effective UID = %d, GID = %d.\n",
		getuid(), getgid(), geteuid(), getegid());

	if (al)
		printf("Alarm set to go off in %u seconds.\n", al);

	printf("File Descriptors:\n");
	for (i=0; i!=40; i++)	/* I hope 40 is enough. */
		if (fstat(i, &sb) == -1) {
			if (errno != EBADF)
				printf("\t%d error: %s\n", i, strerror(errno));
		} else {
			printf("\t%d: dev=%d, ino=%d, perm=0%04o, ",
				i, sb.st_dev, sb.st_ino, sb.st_mode&07777);
#ifdef S_IFMT	/* Posix doesn't specify this. */
			switch (sb.st_mode & S_IFMT) {
# ifdef S_IFREG
			case S_IFREG:
				printf("pipe or regular file\n");
				break;
# endif
# ifdef S_IFDIR
			case S_IFDIR:
				puts("directory");
				break;
# endif
# ifdef S_IFCHR
			case S_IFCHR:
				puts("character device");
				break;
# endif
# ifdef S_IFBLK
			case S_IFBLK:
				puts("block device");
				break;
# endif
# ifdef S_IFIFO
			case S_IFIFO:
				puts("fifo");
				break;
# endif
# ifdef S_IFLNK
			case S_IFLNK:
				puts("symlink");
				break;
# endif
# ifdef S_IFSOCK
			case S_IFSOCK:
				puts("socket");
				break;
# endif
			/* extend as desired */
			default:
				printf("IFMT=0%o\n", sb.st_mode>>12);
				break;
			}
#endif
		}

	printf("Signals:\n");
	for (i=1; i!=40; i++) {	/* I hope 40 is enough. */
		register long n = (long) signal(i, SIG_IGN);
		if (n == (long)SIG_IGN) {
			printf("\t%d: SIG_IGN\n", i);
		} else if (n != -1 && n != (long)SIG_DFL) {
			printf("\t%d: %lu\n", i, (unsigned long)n);
		}
	}

	printf("Environment:\n");
	for (i=0; envp[i]!=NULL; i++)
		printf("\t\"%s\"\n", envp[i]);

	exit(0);
}
